SystemOrganization addCategory: #'AL-Http-Core'!
SystemOrganization addCategory: #'AL-Http-Values'!
SystemOrganization addCategory: #'AL-Http-Exceptions'!
SystemOrganization addCategory: #'AL-Http-Endpoints'!
SystemOrganization addCategory: #'AL-Http-Protocol'!
SystemOrganization addCategory: #'AL-Http-Relays'!

!Collection methodsFor: '*al-http-converting' stamp: 'cwp 7/13/2012 13:08'!
asCookieSpecs
	^ ALCookieSpecs withAll: self! !

!String methodsFor: '*al-http-dispatch' stamp: 'cwp 7/13/2012 18:34'!
printWith: aPrinter
	aPrinter printString: self! !

Object subclass: #ALClient
	instanceVariableNames: 'host port in out handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:38'!
host: aString
	^ self host: aString port: 80 ! !

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/4/2011 00:09'!
host: aString port: anInteger
	^ self host: aString port: anInteger in: nil out: nil! !

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/4/2011 00:08'!
host: aString port: anInteger in: aReadStream out: aWriteStream
	^ self basicNew
		initializeWithHost: aString
		port: anInteger
		in: aReadStream
		out: aWriteStream! !

!ALClient methodsFor: 'private' stamp: 'cwp 9/6/2011 17:11'!
addHostToRequest: aRequest
	| ws |
	ws := String new writing.
	ws write: host.
	port = 80 ifFalse:
		[ws put: $:.
		ws print: port].
	aRequest host: ws conclusion! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/12/2011 22:28'!
addRelay: aRelay
	aRelay destination: handler.
	handler := aRelay! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 16:13'!
connect
	self isConnected ifFalse:
		[| sock |
		Socket initializeNetwork.
		sock := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
		sock connectToHostNamed: host port: port.
		in := sock reading.
		out := sock writing]! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 16:32'!
disconnect
	in ifNotNil:
		[in close.
		in := nil].
	out ifNotNil:
		[out close.
		out := nil]! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/6/2011 15:50'!
get: aPath
	| request response |
	request := ALRequest get: aPath.
	response := self sendRequest: request.
	^ response content! !

!ALClient methodsFor: 'public' stamp: 'cwp 10/24/2011 15:19'!
handleRequest: aRequest
	| parser response printer |
	self connect.
	printer := ALHttpPrinter on: out.
	parser := ALHttpParser on: in.
	printer printRequest: aRequest.
	response := parser parseResponse.
	response body body: [in reading].
	^ response! !

!ALClient methodsFor: 'accessing' stamp: 'cwp 9/4/2011 00:10'!
host
	^ host! !

!ALClient methodsFor: 'initialize-release' stamp: 'cwp 9/11/2011 15:10'!
initializeWithHost: aString port: anInteger in: aReadStream out: aWriteStream
	self initialize.
	host := aString.
	port := anInteger.
	in := aReadStream.
	out := aWriteStream.
	handler := ALChunkingRelay destination: self.
	handler := ALLimitingRelay destination: handler.
	handler := ALResponseDecodingRelay destination: handler.
	handler := ALHostRelay destination: handler host: host port: port.
! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 17:03'!
isConnected
	in ifNil: [^ false].
	out ifNil: [^ false].
	^ true! !

!ALClient methodsFor: 'accessing' stamp: 'cwp 9/4/2011 00:10'!
port
	^ port! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/7/2011 13:16'!
sendRequest: aRequest
	^ handler handleRequest: aRequest! !

Object subclass: #ALFormDataParser
	instanceVariableNames: 'in out'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALFormDataParser class methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 15:02'!
in: aReadStream out: aWriteStream
	^ self basicNew initializeWithIn: aReadStream out: aWriteStream! !

!ALFormDataParser class methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 15:04'!
parse: aReadable
	^ (self in: aReadable reading percentDecoding out: Array new writing) parse! !

!ALFormDataParser methodsFor: 'as yet unclassified' stamp: 'cwp 9/24/2011 15:16'!
assert: aCharacter
	in get = aCharacter ifFalse: 
		[ALParseError signal: 'Expected ', aCharacter asString]! !

!ALFormDataParser methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 15:03'!
initializeWithIn: aReadStream out: aWriteStream
	self initialize.
	in := aReadStream.
	out := aWriteStream.! !

!ALFormDataParser methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 15:05'!
parse
	[[out put: self parseAssociation.
	self assert: $& ] repeat ]
		on: Incomplete
		do: [ :ignore |  ].
	^ out conclusion! !

!ALFormDataParser methodsFor: 'as yet unclassified' stamp: 'cwp 9/24/2011 14:53'!
parseAssociation
	| key value |
	key := in get.
	self assert: $=.
	value := in get.
	^ key -> value! !

Object subclass: #ALHost
	instanceVariableNames: 'name port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Values'!

!ALHost class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:34'!
name: aString
	^ self name: aString port: nil! !

!ALHost class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:00'!
name: aString port: anInteger
	^ self basicNew initializeWithName: aString port: anInteger! !

!ALHost class methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 14:05'!
parseWith: aParser
	^ aParser parseHost! !

!ALHost methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:01'!
initializeWithName: aString port: anInteger
	self initialize.
	name := aString.
	port := anInteger! !

!ALHost methodsFor: 'accessing' stamp: 'cwp 10/25/2011 14:02'!
name
	^ name! !

!ALHost methodsFor: 'accessing' stamp: 'cwp 10/25/2011 14:34'!
port
	^ port ! !

!ALHost methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:36'!
portDo: aBlock
	port ifNotNil: [aBlock value: port]! !

!ALHost methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 14:35'!
printWith: aPrinter
	aPrinter printHost: self! !

Object subclass: #ALHttpInterpreter
	instanceVariableNames: ''
	classVariableNames: 'CarriageReturn Colon Comma Digits Equals LineFeed Semicolon Slash Space Token Version'
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpInterpreter class methodsFor: 'as yet unclassified' stamp: 'cwp 7/12/2012 16:06'!
initialize
	"Initialize character classes with their ASCII values"

	CarriageReturn := 13.
	Colon := 58.
	Comma := 44.
	Equals := 61.
	LineFeed := 10.
	Semicolon := 59.
	Slash := 47.
	Space := 32.
	Digits := #[48 49 50 51 52 53 54 55 56 57] asIdentitySet.	
	Version := Digits copyWith: 46.
	Token := #[33 35 36 37 38 39 42 43 45 46 48 49 50 51 52 53 54 55 56 57 
				65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 
				85 86 87 88 89 90 94 95 96 97 98 99 100 101 102 103 104 105 
				106 107 108 109 110 111 112 113 114 115 116 117 118 119 120
				121 122 124 126] asIdentitySet.
	
	
! !

ALHttpInterpreter subclass: #ALHttpParser
	instanceVariableNames: 'in ch result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 9/15/2011 15:44'!
on: aReadStream
	^ self basicNew initializeWithStream: aReadStream! !

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:21'!
parse: aReadable as: aClass
	^ (self on: aReadable reading) parse: aClass! !

!ALHttpParser class methodsFor: 'parsing' stamp: 'cwp 7/11/2012 22:04'!
parseCookieValues: aReadStream
	^ (self on: aReadStream) parseCookieValues! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 9/15/2011 15:41'!
assert: aByte
	ch = aByte ifFalse: [ALMalformedRequest signal].
	ch := nil.! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 9/15/2011 15:41'!
assertCrLf
	"Character cr asciiValue = 13"
	"Character lf acsiiValue = 10"
	
	self assert: 13.
	self next: 10.
! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 9/15/2011 15:41'!
assertSpace
	"Character space asciiValue = 32"
	self assert: 32.
! !

!ALHttpParser methodsFor: 'initialize-release' stamp: 'cwp 9/15/2011 15:35'!
initializeWithStream: aReadStream
	in := aReadStream
	! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 9/15/2011 15:41'!
next: expected
	| actual |
	actual := in get.
	actual = expected ifFalse: [ALMalformedRequest signal]! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 10/24/2011 15:02'!
parse: aClass
	^ aClass parseWith: self! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 10/24/2011 15:01'!
parseContentType
	result := self parseTypeSubtype.
	self parseParameters.
	^ result! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 7/12/2012 22:54'!
parseCookieSpec
	| name value |
	name := self parseToken.
	self assert: Equals.
	value := self parseTokenUntilSemicolon.
	result := ALCookie name: name value: value.
	self parseParameters.
	^ result! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 7/12/2012 23:20'!
parseCookieSpecs
	"Parsing cookies on the client side is... complex. See RFC 6265."
	self shouldBeImplemented.
"	out := (Array new: 1) writing.
	[out put: self parseCookieSpec.
	[true] whileTrue: 
		[self assert: Comma.
		self next: Space.
		out put: self parseCookieSpec]]
			on: Incomplete
			do: [:nothing].
	^ out conclusion"! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 7/11/2012 22:30'!
parseCookieValue
	| name value |
	name := self parseToken.
	self assert: Equals.
	value := self parseTokenUntilSemicolon.
	^ ALCookie name: name value: value! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 7/11/2012 22:25'!
parseCookieValues
	| out |
	out := (Array new: 1) writing.
	[out put: self parseCookieValue.
	[true] whileTrue: 
		[self assert: Semicolon.
		self next: Space.
		out put: self parseCookieValue]]
			on: Incomplete
			do: [:nothing].
	^ out conclusion ! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:02'!
parseFieldValue
	"Character tab asciiValue = 9"
	"Character space asciiValue = 32"
	"Character cr asciivalue = 13"
	"Character lf asciiValue = 10"
	"For now, we ignore the possibility of line folding"

	| out byte |
	[byte := in get.
	byte = 32 or: [byte = 9]] whileTrue.
	out := ByteArray new writing.
	out put: byte.
	[(byte := in get) = CarriageReturn] whileFalse: [out put: byte].
	self next: LineFeed.
	^ out conclusion! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 7/13/2012 18:31'!
parseHeaders
	
	| name value |
	[name := self parseToken.
	name isEmpty ifTrue: [^ self assertCrLf].
	self assert: Colon.
	value := self parseFieldValue.
	result rawHeader: name value: value]
		repeat
	
! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 14:10'!
parseHost
	| name port token |
	name := self parseToken.
	ch = Colon ifTrue:
		[token := self parseToken.
		port := XTNumberParser 
			parse: token asString 
			base: 10 
			ifFail: [ALParseError signal: 'Expected a port number']].
	^ ALHost name: name port: port! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:02'!
parseMethod
	| method |
	method := self parseToken.
	ch ifNil: [Incomplete signal].
	self assertSpace.
	^ method! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 7/12/2012 22:52'!
parseParameter
 
	| attribute value |
	in available = 0 ifTrue: [^ false].
	ch = Comma ifTrue: [^ false].
	self assert: Semicolon.
	self next: Space.
	attribute := self parseToken.
	self assert: Equals.
	value := self parseToken.
	result parameter: attribute value: value.
	^ true! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:01'!
parseParameters
	
	[[self parseParameter] whileTrue]
		on: Incomplete
		do: [:error]! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:02'!
parsePath

	| out path |
	path := ALPath root.
	ch := in get. 
	ch = Slash ifFalse: [self error].

	out := String new writing.
	[(ch := in get) = Space] whileFalse:
		[ch = Slash
			ifFalse: [out put: (Character value: ch)]
			ifTrue: 
				[path := path / out conclusion.
				out := String new writing]].
	path := path / out conclusion.
	self assertSpace.
	^ path
			! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:02'!
parseReason
	^ (in ending: 13) rest asString! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 10/24/2011 15:02'!
parseRequest
	result := self parseRequestLine.
	self parseHeaders.
	^ result! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/8/2012 00:04'!
parseRequestLine
	| version path method entity |
	method := self parseMethod.
	path := self parseUrl.
	version := self parseVersion.
	self assertCrLf.
	entity := ALEntity empty.
	^ ALRequest method: method url: path version: version body: entity.
	! !

!ALHttpParser methodsFor: 'dispatch' stamp: 'cwp 10/24/2011 15:02'!
parseResponse
	result := self parseStatusLine.
	self parseHeaders.
	^ result! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:02'!
parseStatus
	| digits |
	digits := self readAll: Digits.
	^ Integer readFrom: digits readStream! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:02'!
parseStatusLine
	| version reason status body |
	version := self parseVersion.
	self assert: Space.
	status := self parseStatus.
	self assert: Space.
	reason := self parseReason.
	self next: LineFeed.
	body := ALEntity empty.
	^ ALResponse status: status reason: reason version: version body: body! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/15/2011 15:37'!
parseToken

	|  out |
	out := ByteArray new writing.
	ch ifNotNil: [(Token includes: ch) ifTrue: [out put: ch]].
	[[Token includes: (ch := in get)] whileTrue: [out put: ch]]
		on: Incomplete
		do: [:error].
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 7/11/2012 22:35'!
parseTokenUntilSemicolon

	|  out |
	out := ByteArray new writing.
	ch ifNotNil: [Semicolon = ch ifFalse: [out put: ch]].
	[[Semicolon = (ch := in get)] whileFalse: [out put: ch]]
		on: Incomplete
		do: [:error | ch := Semicolon].
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:01'!
parseTypeSubtype

	| type subtype |
	type := self parseToken.
	self assert: Slash.
	subtype := self parseToken.
	^ ALContentType type: type subtype: subtype! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/4/2012 14:53'!
parseUrl
	| rs ws |
	rs := ((in ending: 32) encoding: #ascii) percentDecoding.
	ws := ALUrl writing.
	ws write:  rs.
	^ ws conclusion! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 10/24/2011 15:03'!
parseVersion
	| version |
	self next: $H asciiValue.
	self next: $T asciiValue.
	self next: $T asciiValue.
	self next: $P asciiValue.
	self next: Slash.
	version := self readAll: Version.
	^ version! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/15/2011 15:38'!
readAll: aByteArray
	| out |
	out := ByteArray new writing.
	[aByteArray includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

ALHttpInterpreter subclass: #ALHttpPrinter
	instanceVariableNames: 'out message ascii'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpPrinter class methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 15:54'!
on: aWriteStream
	^ self basicNew initializeWithStream: aWriteStream ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:20'!
crlf

	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'initialize-release' stamp: 'cwp 10/25/2011 14:54'!
initializeWithStream: aWriteStream
	out := aWriteStream.
	ascii := aWriteStream encoding: #ascii.
	ascii setLineEndTransparent ! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 10/19/2011 19:36'!
print: aMessage
	aMessage printWith: self! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 14:22'!
printBytes: aByteArray 
	out write: aByteArray! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 7/12/2012 15:42'!
printCommaSequence: aCollection
	aCollection
		do: [:ea | ea printWith: self]
		separatedBy: [out write: ', ']! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 7/12/2012 15:47'!
printContentType: aContentType
	out
		write: aContentType type;
		put: Slash;
		write: aContentType subtype.
	self printParametersOf: aContentType! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 7/12/2012 15:47'!
printCookie: aCookie 
	out 
		write: aCookie name;
		put: $=;
		write: aCookie value.
	self printParametersOf: aCookie! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 10/25/2011 14:50'!
printHeader: name value: value
	out
		write: name;
		put: Colon;
		put: Space.
	self print: value.
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/8/2011 11:11'!
printHeaders
	message rawHeadersDo: [ :name :value | self printHeader: name value: value ].
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 14:52'!
printHost: aHost
	out write: aHost name.
	aHost portDo:
		[:port |
		out put: Colon.
		ascii print: port]! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 15:00'!
printNumber: aNumber
	ascii print: aNumber! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 7/12/2012 15:47'!
printParametersOf: aContentType
	aContentType
		parametersDo: [ :attribute :value | 
			out
				put: Semicolon;
				put: Space;
				write: attribute;
				put: Equals;
				write: value ]! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:39'!
printRequest
	self printRequestLine.
	self printHeaders.
	out flush! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 9/4/2011 01:30'!
printRequest: aRequest 
	message := aRequest.
	self printRequest.
	message := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 10/8/2012 14:08'!
printRequestLine

	out
		write: message method;
		put: Space.
	self printUrl: message url.
	out put: Space.
	self printVersion.
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:26'!
printResponse
	self printStatusLine.
	self printHeaders.
	out flush! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 9/4/2011 01:30'!
printResponse: aResponse
	message := aResponse.
	self printResponse.
	message := nil! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 7/12/2012 15:51'!
printSemicolonSequence: aCollection
	aCollection
		do: [:ea | ea printWith: self]
		separatedBy: [out write: '; ']! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 10/25/2011 14:52'!
printStatusLine
	self printVersion.
	out space.
	ascii print: message status.
	out
		space;
		write: message reason;
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'dispatch' stamp: 'cwp 7/13/2012 18:35'!
printString: aString
	out write: aString! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 10/8/2012 14:14'!
printUrl: aUrl
	aUrl streamingPrintOn: ascii! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:35'!
printVersion
	out
		write: 'HTTP/';
		write: (String streamContents: [ :s | message version printFractionAsDecimalOn: s ])! !

Object subclass: #ALMimeMessage
	instanceVariableNames: 'headers body relay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

ALMimeMessage subclass: #ALEntity
	instanceVariableNames: 'filters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALEntity class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 16:18'!
body: anObject
	^ self basicNew initializeWithBody: anObject! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 9/8/2011 14:46'!
callback: aBlock
	^ self body: aBlock! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 9/8/2011 14:46'!
empty
	^ ALEntity body: nil! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 7/13/2012 18:31'!
html: aString
	^ (self callback: [:out | (out encoding: #ascii) write: aString])
		header: 'Content-Type' value: 'text/html';
		header: 'Content-Length' value: aString size printString;
		yourself! !

!ALEntity class methodsFor: 'reading' stamp: 'cwp 9/8/2011 14:47'!
reading: aReadable
	^ self body: [aReadable reading]! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 10/25/2011 13:37'!
contentLength
	^ self parseHeader: 'Content-Length' with: [:bytes | Integer readFrom: bytes asString readStream]! !

!ALEntity methodsFor: 'headers - setting' stamp: 'cwp 10/25/2011 13:37'!
contentLength: anInteger 
	headers at: 'Content-Length' put: anInteger! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 10/25/2011 13:23'!
contentType
	^ self parseHeader: 'Content-Type' as: ALContentType.! !

!ALEntity methodsFor: 'headers - setting' stamp: 'cwp 10/25/2011 13:38'!
contentType: aContentType 
	headers at: 'Content-Type' put: aContentType! !

!ALEntity methodsFor: 'accessing' stamp: 'cwp 9/7/2011 15:16'!
filter
	| catcher |
	catcher := MessageCatcher new.
	catcher privAccumulator: filters.
	^ catcher
		! !

!ALEntity methodsFor: 'initialize-release' stamp: 'cwp 9/7/2011 15:06'!
initializeWithBody: anObject
	super initializeWithBody: anObject.
	filters := OrderedCollection new    ! !

!ALEntity methodsFor: 'testing' stamp: 'cwp 10/25/2011 15:08'!
isEmpty
	^ headers isEmpty! !

!ALEntity methodsFor: 'io' stamp: 'cwp 9/12/2011 17:25'!
readWith: aBlock
	| stream |
	body ifNil: [^ nil].
	stream := filters
		inject: body value
		into: [:s :message | message sendTo: s].
	^ [aBlock value: stream] ensure: 
		[(filters anySatisfy: [:m | m selector = #limiting:]) ifTrue: [stream rest]]  ! !

!ALEntity methodsFor: 'io' stamp: 'cwp 7/13/2012 23:18'!
writeBodyInto: aWriteStream
	| stream |
	body ifNil: [^ nil].
	stream := filters reversed
		inject: aWriteStream
		into: [:s :message | message sendTo: s].
	^ [body value: stream]
		ensure: [stream close].
! !

ALMimeMessage subclass: #ALHttpMessage
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALHttpMessage class methodsFor: 'constants' stamp: 'cwp 9/5/2011 14:20'!
defaultVersion
	^ 1.1s1! !

!ALHttpMessage class methodsFor: 'instance creation' stamp: 'cwp 10/19/2011 19:41'!
readFrom: aReadStream
	| parser message |
	parser := ALHttpParser on: aReadStream.
	message := parser parse: self.
	message readEntityFrom: aReadStream.
	^ message! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 18:31'!
beConnectionClose
	self rawHeader: 'Connection' value:  #[99 108 111 115 101]
! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 18:31'!
beConnectionKeepAlive
	self rawHeader: 'Connection' value: #[107 101 101 112 45 97 108 105 118 101] ! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 18:31'!
beTransferEncodingChunked
	self rawHeader: 'Transfer-Encoding' value: #[99 104 117 110 107 101 100]! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 10/14/2011 23:59'!
consume
	self readEntityWith: [:in | in rest].  ! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/6/2011 15:50'!
content
	^ self readEntityWith: [:in | in rest]! !

!ALHttpMessage methodsFor: 'headers - getting' stamp: 'cwp 9/8/2011 23:46'!
contentLength
	^ body contentLength! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 23:46'!
contentLength: anInteger
	^ body contentLength: anInteger! !

!ALHttpMessage methodsFor: 'headers - getting' stamp: 'cwp 9/8/2011 14:29'!
contentType
	^ body contentType! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 14:29'!
contentType: anObject
	^ body contentType: anObject! !

!ALHttpMessage methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:11'!
initializeWithVersion: aScaledDecimal body: anEntity
	self initializeWithBody: anEntity.
	version := aScaledDecimal ! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 9/8/2011 14:57'!
isConnectionClose
	^ (self rawHeader: 'Connection' ifAbsent: [^ false]) = #[99 108 111 115 101]! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 9/8/2011 15:01'!
isConnectionKeepAlive
	"'keep-alive' asByteArray"
	
	^ (self rawHeader: 'Connection' ifAbsent: [^ true]) 
		= #[107 101 101 112 45 97 108 105 118 101]! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 5/2/2011 18:37'!
isTransferEncodingChunked
	^ (self header: 'Transfer-Encoding') = 'chunked'! !

!ALHttpMessage methodsFor: 'headers - getting' stamp: 'cwp 10/8/2012 00:16'!
location
	"This is a hack until proper URI parsing is implemented."
	
	^ self parseHeader: 'Location' with:
		[:raw || bytes |
		bytes := raw copyWith: 32.
		ALHttpParser parse: bytes reading as: ALUrl]! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 18:36'!
location: aPath
	self header: 'Location' value: aPath ! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:56'!
rawHeader: aString ifAbsent: aBlock
	^ (self class entityHeaders includes: aString)
		ifTrue: [body rawHeader: aString ifAbsent: aBlock]
		ifFalse: [headers at: aString ifAbsent: aBlock]! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 7/13/2012 18:31'!
rawHeader: aString value: aByteArray
	(self class entityHeaders includes: aString)
		ifTrue: [body rawHeader: aString value: aByteArray]
		ifFalse: [headers at: aString put: aByteArray]! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:11'!
rawHeadersDo: aBlock
	super rawHeadersDo: aBlock.
	body rawHeadersDo: aBlock.! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 10/25/2011 15:08'!
readEntityFrom: aReadStream
	body isEmpty ifFalse: [body body: [aReadStream]]! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/6/2011 15:51'!
readEntityWith: aBlock
	^ body readWith: aBlock! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version
	^ version! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version: anObject
	version := anObject! !

!ALHttpMessage methodsFor: 'io' stamp: 'cwp 7/13/2012 23:18'!
writeBodyInto: aStream
	| printer |
	printer := ALHttpPrinter on: aStream.
	printer print: self.
	self body writeInto: aStream! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/8/2011 23:48'!
writeEntityWith: aBlock
	body body: aBlock! !

ALHttpMessage subclass: #ALRequest
	instanceVariableNames: 'method url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALRequest class methodsFor: 'convenience' stamp: 'cwp 10/8/2012 00:05'!
get: aUrl 
	^ (self method: 'GET' url: aUrl)
		userAgent: self userAgent;
		acceptAnything;
		yourself! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 10/8/2012 00:05'!
method: aString url: aUrl
	^ self
		method: aString
		url: aUrl
		version: self defaultVersion
		body: ALEntity empty! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 10/8/2012 00:05'!
method: aString url: aUrl version: aScaledDecimal 
	^ self 
		method: aString 
		url: aUrl 
		version: aScaledDecimal 
		body: ALEntity empty! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 10/8/2012 00:04'!
method: aString url: aUrl version: aScaledDecimal body: anEntity
	^ self basicNew
		initializeWithMethod: aString 
		url: aUrl 
		version: aScaledDecimal 
		body: anEntity ! !

!ALRequest class methodsFor: 'dispatch' stamp: 'cwp 10/19/2011 19:42'!
parseWith: aParser
	^ aParser parseRequest! !

!ALRequest class methodsFor: 'accessing' stamp: 'cwp 9/4/2011 02:01'!
userAgent
	^ 'Altitude/0.1'! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 18:31'!
accept: aString
	self header: 'Accept' value: aString! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:00'!
acceptAnything
	self accept: '*/*'! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 20:28'!
addCookie: name value: value
	| cookie values |
	cookie := ALCookie name: name value: value.
	values := self cookie
		ifNil: [ALCookieValues with: cookie]
		ifNotNil: [:collection | collection copyWith: cookie].
	self header: 'Cookie' value: values! !

!ALRequest methodsFor: 'headers - getting' stamp: 'cwp 7/13/2012 20:24'!
cookie
	^ self parseHeader: 'Cookie' as: ALCookieValues! !

!ALRequest methodsFor: 'headers - getting' stamp: 'cwp 7/13/2012 20:12'!
cookiesDo: aBlock
	| cookies |
	cookies := self cookie ifNil: [^ self].
	cookies do: aBlock! !

!ALRequest methodsFor: 'headers - getting' stamp: 'cwp 10/25/2011 14:05'!
host
	^ self parseHeader: 'Host' as: ALHost! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 10/25/2011 14:03'!
host: aHost
	headers at: 'Host' put: aHost! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 10/8/2012 00:04'!
initializeWithMethod: aString url: aUrl version: aScaledDecimal body: anEntity
	self initializeWithVersion: aScaledDecimal body: anEntity.
	method := aString.
	url := aUrl.
	! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:23'!
isMethodConnect
	^ method = 'CONNECT'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:23'!
isMethodDelete
	^ method = 'DELETE'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 3/21/2011 22:25'!
isMethodGet
	^ method = 'GET'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:23'!
isMethodHead
	^ method = 'HEAD'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:24'!
isMethodOptions
	^ method = 'OPTIONS'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:24'!
isMethodPost
	^ method = 'POST'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:24'!
isMethodPut
	^ method = 'PUT'! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 10/1/2011 16:24'!
isMethodTrace
	^ method = 'TRACE'! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:01'!
method
	^ method! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 10/7/2012 23:32'!
path
	^ url path! !

!ALRequest methodsFor: 'dispatch' stamp: 'cwp 10/19/2011 19:37'!
printWith: aPrinter
	aPrinter printRequest: self! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 10/8/2012 00:03'!
url
	^ url! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 18:31'!
userAgent: aString
	self header: 'User-Agent' value: aString! !

ALHttpMessage subclass: #ALResponse
	instanceVariableNames: 'status reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 7/13/2012 20:05'!
forbidden
	^ self status: 403 reason: 'Forbidden' details: 'You have not logged in'! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 9/30/2011 22:59'!
notFound: aPath
	^ self 
		status: 404 
		reason: 'Not Found'
		details: aPath asString, ' does not exist on this server'! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 9/9/2011 00:06'!
ok
	^ self status: 200 reason: 'OK'! !

!ALResponse class methodsFor: 'dispatch' stamp: 'cwp 10/19/2011 19:42'!
parseWith: aParser
	^ aParser parseResponse! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 7/13/2012 18:31'!
seeOther: aPath
	^ (self status: 303 reason: 'See Other')
		header: 'Location' value: aPath printString;
		yourself! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:17'!
status: anInteger reason: aString
	^ self 	
		status: anInteger 
		reason: aString 
		version: self defaultVersion 
		body: ALEntity empty! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:18'!
status: anInteger reason: aString body: anEntity
	^ self 
		status: anInteger 
		reason: aString 
		version: self defaultVersion  
		body: anEntity! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 9/30/2011 23:16'!
status: status reason: aString details: aString2
	| html |
	html := String new writing
		write: '<!!DOCTYPE html><html><head><title>';
		print: status;
		space;
		write: aString;
		write: '</title></head><body><h1>';
		write: aString;
		write: '</h1><p>';
		write: aString2;
		write: '</p></body></html>';
		conclusion.
	^ self status: status reason: aString body: (ALEntity html: html)! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:18'!
status: anInteger reason: aString version: aScaledDecimal
	^ self 
		status: anInteger 
		reason: aString 
		version: aScaledDecimal 
		body: ALEntity empty! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:14'!
status: anInteger reason: aString version: aScaledDecimal body: anEntity
	^ self basicNew
		initializeWithStatus: anInteger 
		reason: aString
		version: aScaledDecimal
		body: anEntity! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 5/2/2011 18:27'!
type: aString callback: aBlock
	| entity |
	entity := ALEntity callback: aBlock.
	entity contentType: aString.
	^ (self status: 200 reason: 'OK' body: entity)
		beTransferEncodingChunked;
		yourself! !

!ALResponse methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:29'!
initializeWithStatus: anInteger reason: aString version: aScaledDecimal body: anEntity
	self initializeWithVersion: aScaledDecimal body: anEntity.
	status := anInteger.
	reason := aString.
	! !

!ALResponse methodsFor: 'dispatch' stamp: 'cwp 10/19/2011 19:37'!
printWith: aPrinter
	aPrinter printResponse: self! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
reason
	^ reason! !

!ALResponse methodsFor: 'headers - getting' stamp: 'cwp 7/13/2012 20:24'!
setCookie
	^ self parseHeader: 'Set-Cookie' as: ALCookieSpecs.! !

!ALResponse methodsFor: 'headers - setting' stamp: 'cwp 7/13/2012 20:24'!
setCookie: aCookieOrSpecs
	| specs |
	specs := aCookieOrSpecs class == ALCookieSpecs
		ifTrue: [aCookieOrSpecs]
		ifFalse: [ALCookieSpecs with: aCookieOrSpecs].
	self header: 'Set-Cookie' value: specs.! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
status
	^ status! !

!ALMimeMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 14:45'!
entityHeaders
	^ #(
		'Content-Encoding' 
		'Content-Language'
		'Content-Length'
		'Content-Location'
		'Content-MD5'
		'Content-Range'
		'Content-Type'
		'Expires'
		'Last-Modified'
	)! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 7/13/2012 23:28'!
addRelay: aBlock
	relay := aBlock value: relay! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 9/7/2011 14:47'!
body
	^ body! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 5/2/2011 18:33'!
body: anObject
	 body := anObject! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:06'!
header: aString
	^ (self rawHeader: aString) ifNotNil: [:value | value asString]! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 7/13/2012 18:32'!
header: aString value: anObject
	self rawHeader: aString value: anObject! !

!ALMimeMessage methodsFor: 'initialize-release' stamp: 'cwp 7/13/2012 23:07'!
initializeWithBody: anObject
	self initialize.
	headers := Dictionary new.
	body := anObject.
	relay := ALTerminalRelay new! !

!ALMimeMessage methodsFor: 'parsing' stamp: 'cwp 10/25/2011 13:36'!
parseHeader: aString as: aClass
	^ self parseHeader: aString with: [:value | ALHttpParser parse: value as: aClass]! !

!ALMimeMessage methodsFor: 'parsing' stamp: 'cwp 7/11/2012 21:32'!
parseHeader: aString with: aBlock
	^ headers
		at: aString
		ifPresent:
			[:value |
			value class == ByteArray
				ifTrue: [headers at: aString put: (aBlock value: value)]
				ifFalse: [value]]! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:56'!
rawHeader: aString
	^ self rawHeader: aString ifAbsent: [ nil ]! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:54'!
rawHeader: aString ifAbsent: aBlock
	^ headers at: aString ifAbsent: aBlock! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 10/25/2011 13:13'!
rawHeader: aString ifPresent: pBlock ifAbsent: aBlock
	^ headers at: aString ifPresent: pBlock ifAbsent: aBlock! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 7/13/2012 18:31'!
rawHeader: aString value: aByteArray
	headers at: aString put: aByteArray! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:11'!
rawHeadersDo: aBlock
	headers keysAndValuesDo: aBlock  ! !

!ALMimeMessage methodsFor: 'io' stamp: 'cwp 7/13/2012 23:17'!
writeInto: aWriteStream
	relay write: self into: aWriteStream! !

Object subclass: #ALParameterizedValue
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Values'!

ALParameterizedValue subclass: #ALContentType
	instanceVariableNames: 'type subtype'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Values'!

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 12:59'!
ascii
	^ self type: 'text' subtype: 'plain' charset: 'ascii'
	! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 12:59'!
css
	^ self type: 'text' subtype: 'css' charset: 'UTF-8'! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 13:02'!
favicon
	^ self type: 'image' subtype: 'vnd.microsoft.icon'! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 13:00'!
html
	^ self type: 'text' subtype: 'html' charset: 'UTF-8'
	
		! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 13:03'!
javascript
	^ self type: 'application' subtype: 'javascript' charset: 'UTF-8'! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 13:00'!
json
	^ self type: 'application' subtype: 'json' charset: 'UTF-8'! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 13:03'!
less
	^ self type: 'text' subtype: 'x-lesscss' charset: 'UTF-8'! !

!ALContentType class methodsFor: 'dispatch' stamp: 'cwp 10/24/2011 14:14'!
parseWith: aParser
	^ aParser parseContentType! !

!ALContentType class methodsFor: 'common types' stamp: 'cwp 10/25/2011 13:04'!
png
	^ self type: 'image' subtype: 'png' ! !

!ALContentType class methodsFor: 'instance creation' stamp: 'cwp 9/7/2011 15:56'!
type: typeString subtype: subtypeString
	^ self basicNew
		initializeWithType: typeString subtype: subtypeString! !

!ALContentType class methodsFor: 'instance creation' stamp: 'cwp 10/25/2011 12:58'!
type: type subtype: subtype charset: charset
	^ (self type: type subtype: subtype)
		charset: charset
		! !

!ALContentType methodsFor: 'comparing' stamp: 'cwp 10/22/2011 12:28'!
= other
	self species = other species ifFalse: [^ false].
	self type = other type ifFalse: [^ false].
	self subtype = other subtype ifFalse: [^ false].
	self parametersDo:
		[:name :value || expected |
		expected := other parameter: name ifAbsent: [^ false].
		value = expected ifFalse: [^ false]].
	^ true! !

!ALContentType methodsFor: 'converting' stamp: 'cwp 7/11/2012 22:44'!
asByteArray
	| ws |
	ws := ByteArray new writing.
	ws
		write: type;
		put: $/ asciiValue;
		write: subtype.
	self writeParametersOn: ws.
	^ ws conclusion! !

!ALContentType methodsFor: 'parameters' stamp: 'cwp 9/8/2011 14:15'!
charset
	^ (self parameter: 'charset' ifAbsent: [^ nil]) asLowercase asSymbol! !

!ALContentType methodsFor: 'parameters' stamp: 'cwp 9/8/2011 14:29'!
charset: aString
	self parameter: 'charset' value: aString asSymbol! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/8/2011 14:18'!
hasCharset
	^ parameters includesKey: 'charset' ! !

!ALContentType methodsFor: 'comparing' stamp: 'cwp 10/22/2011 12:32'!
hash
	| hash |
	hash := String stringHash: type initialHash: self class hash.
	hash := String stringHash: subtype initialHash: hash.
	self parametersDo:
		[:name :value |
		hash := String stringHash: name initialHash: hash.
		hash := String stringHash: value initialHash: hash].
	^ hash
	! !

!ALContentType methodsFor: 'initialize-release' stamp: 'cwp 9/7/2011 15:59'!
initializeWithType: typeString subtype: subtypeString
	self initialize.
	type := typeString asLowercase.
	subtype := subtypeString asLowercase.
	parameters := Dictionary new.! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:01'!
isHtml
	^ self isText and: [subtype = 'html']! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:09'!
isImage
	^ type = 'image'! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:10'!
isJpeg
	^ self isImage and: [subtype = 'jpeg']! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 3/31/2012 22:39'!
isJson
	^ type = 'application' and: [subtype = 'json']! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:01'!
isText
	^ self type = 'text'! !

!ALContentType methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 14:23'!
printWith: aPrinter
	aPrinter printContentType: self  ! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:00'!
subtype
	^ subtype ! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:00'!
type
	^ type! !

ALParameterizedValue subclass: #ALCookie
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Values'!

!ALCookie class methodsFor: 'instance creation' stamp: 'cwp 7/11/2012 22:14'!
name: nameString value: valueString
	^ self name: nameString value: valueString parameters: Dictionary new! !

!ALCookie class methodsFor: 'instance creation' stamp: 'cwp 7/11/2012 22:17'!
name: nameString value: valueString parameters: aDictionary 
	^ self basicNew 
		initializeWithName: nameString
		value: valueString
		parameters: aDictionary! !

!ALCookie methodsFor: 'parameters' stamp: 'cwp 7/11/2012 22:50'!
domain
	^ (self parameter: 'domain' ifAbsent: [^ nil]) asLowercase! !

!ALCookie methodsFor: 'parameters' stamp: 'cwp 7/12/2012 15:22'!
domain: aString
	self parameter: 'domain' value: aString asLowercase! !

!ALCookie methodsFor: 'initialize-release' stamp: 'cwp 7/11/2012 22:17'!
initializeWithName: nameString value: valueString parameters: aDictionary 
	self initialize.
	name := nameString.
	value := valueString.
	parameters := aDictionary! !

!ALCookie methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:47'!
name
	^ name! !

!ALCookie methodsFor: 'parameters' stamp: 'cwp 7/11/2012 22:52'!
path
	self parameter: 'path' ! !

!ALCookie methodsFor: 'parameters' stamp: 'cwp 7/12/2012 15:23'!
path: aPathOrString
	self parameter: 'path' value: aPathOrString asString ! !

!ALCookie methodsFor: 'printing' stamp: 'cwp 7/12/2012 15:43'!
printWith: aPrinter
	aPrinter printCookie: self! !

!ALCookie methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:47'!
value
	^ value! !

!ALParameterizedValue methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:40'!
parameter: aString
	^ self parameter: aString ifAbsent: [nil]! !

!ALParameterizedValue methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:41'!
parameter: aString ifAbsent: aBlock
	^ parameters at: aString ifAbsent: aBlock! !

!ALParameterizedValue methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:40'!
parameter: pString value: vString
	parameters at: pString put: vString! !

!ALParameterizedValue methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:42'!
parameters
	^ parameters! !

!ALParameterizedValue methodsFor: 'accessing' stamp: 'cwp 7/11/2012 22:40'!
parametersDo: aBlock
	parameters keysAndValuesDo: aBlock! !

!ALParameterizedValue methodsFor: 'converting' stamp: 'cwp 7/11/2012 22:45'!
writeParametersOn: aBinaryWriteStream
	parameters keysAndValuesDo: 
		[ :attribute :value | 
		aBinaryWriteStream
			write: #[59 32];
			write: attribute;
			put: 61;
			write: value ]! !

Object subclass: #ALRelay
	instanceVariableNames: 'destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

ALRelay subclass: #ALChunkingRelay
	instanceVariableNames: 'size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALChunkingRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/12/2011 17:23'!
destination: aMessageHandler
	^ self destination: aMessageHandler size: 4096   ! !

!ALChunkingRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:42'!
destination: aMessageHandler size: anInteger
	^ self basicNew
		initializeWithDestination: aMessageHandler 
		size: anInteger  ! !

!ALChunkingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:44'!
annotate: aMessage
	aMessage isTransferEncodingChunked ifTrue:
		[aMessage body filter chunking: size]! !

!ALChunkingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:42'!
initializeWithDestination: aMessageHandler size: anInteger
	self initializeWithDestination: aMessageHandler.
	size := anInteger  ! !

ALRelay subclass: #ALDecodingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALDecodingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 12:19'!
annotate: aMessage
	| body |
	body := aMessage body.
	body contentType ifNotNil: 
		[ :type | 
		type hasCharset ifTrue: 
			[ body filter encoding: type charset ] ].
	
	 ^ aMessage! !

ALDecodingRelay subclass: #ALRequestDecodingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALRequestDecodingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 11:42'!
handleRequest: aRequest
	^ destination handleRequest: (self annotate: aRequest)! !

ALDecodingRelay subclass: #ALResponseDecodingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALResponseDecodingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 11:43'!
handleRequest: aRequest
	^ self annotate: (destination handleRequest: aRequest)! !

ALRelay subclass: #ALErrorRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALErrorRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/30/2011 23:52'!
handleRequest: aRequest
	^ [destination handleRequest: aRequest]
		on: Error
		do: [:error | 
			ALResponse 
				status: 500 
				reason: 'Internal Error' 
				details: error messageText]! !

ALRelay subclass: #ALHostRelay
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALHostRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:31'!
destination: aMessageHandler host: anObject
	| host |
	host := anObject isString
		ifTrue: [ALHost name: anObject]
		ifFalse: [anObject].
	
	^ self basicNew 
		initializeWithDestination: aMessageHandler
		host: host
		! !

!ALHostRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:31'!
destination: aMessageHandler host: aString port: anInteger
	^ self destination: aMessageHandler host: (ALHost name: aString port: anInteger)! !

!ALHostRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 13:16'!
handleRequest: aRequest
	aRequest host: value.
	^ destination handleRequest: aRequest! !

!ALHostRelay methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 14:29'!
initializeWithDestination: aMessageHandler host: aHost
	
	self initializeWithDestination: aMessageHandler.
	value := aHost! !

ALRelay subclass: #ALKeepAliveRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALKeepAliveRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/13/2011 11:13'!
handleRequest: aRequest
	| response |
	response := destination handleRequest: aRequest.
	(aRequest isConnectionClose or: [response isConnectionClose]) ifFalse:
		[response beConnectionKeepAlive.
		response body filter closing: []].
	^ response! !

ALRelay subclass: #ALLimitingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALLimitingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:52'!
annotate: aMessage
	aMessage contentLength ifNotNil: 
		[ :length | aMessage body filter limiting: length ]! !

!ALRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 15:25'!
destination: aMessageHandler
	^ self basicNew initializeWithDestination: aMessageHandler ! !

!ALRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/12/2011 22:19'!
new
	^ self destination: nil! !

!ALRelay methodsFor: 'relaying' stamp: 'cwp 9/11/2011 14:50'!
annotate: aMessage! !

!ALRelay methodsFor: 'accessing' stamp: 'cwp 9/12/2011 22:07'!
destination: aRelay
	destination := aRelay! !

!ALRelay methodsFor: 'relaying' stamp: 'cwp 9/11/2011 14:50'!
handleRequest: aRequest
	| response |
	self annotate: aRequest.
	response := destination handleRequest: aRequest.
	self annotate: response.
	^ response ! !

!ALRelay methodsFor: 'initialization' stamp: 'cwp 9/7/2011 13:13'!
initializeWithDestination: aMessageHandler
	self initialize.
	destination := aMessageHandler! !

!ALRelay methodsFor: 'relaying' stamp: 'cwp 7/13/2012 22:39'!
write: aMessage into: aStream
	destination write: aMessage into: aStream! !

ALRelay subclass: #ALVariableRelay
	instanceVariableNames: 'notification value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALVariableRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:31'!
destination: aRelay notification: aClass value: anObject
	^ self basicNew
		initializeWithDestination: aRelay
		notification: aClass
		value: anObject! !

!ALVariableRelay methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:36'!
handleRequest: aRequest
	[destination handleRequest: aRequest]
		on: notification
		do: [:n | n resume: value]! !

!ALVariableRelay methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:32'!
initializeWithDestination: aRelay notification: aClass value: anObject
	self initializeWithDestination: aRelay.
	notification := aClass.
	value := anObject! !

!ALVariableRelay methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:36'!
write: aMessage into: aStream
	[destination write: aMessage into: aStream]
		on: notification
		do: [:n | n resume: value]! !

Object subclass: #ALResourceQueue
	instanceVariableNames: 'resource queue resourceMutex queueMutex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALResourceQueue class methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2012 14:50'!
for: anObject
	^ self basicNew initializeWithResource: anObject! !

!ALResourceQueue methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2012 01:01'!
add
	queueMutex critical:
		[queue add: Processor activeProcess]! !

!ALResourceQueue methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2012 14:28'!
do: aBlock
	| done first |
	done := false.
	[done] whileFalse:
		[queueMutex critical: [first := queue first].
		first == Processor activeProcess ifTrue:
			[resourceMutex critical: [aBlock value: resource].
			done := true.
			queueMutex critical: [queue removeFirst]]].
	
				! !

!ALResourceQueue methodsFor: 'as yet unclassified' stamp: 'cwp 10/30/2012 01:02'!
initializeWithResource: anObject
	self initialize.
	resource := anObject.
	queue := OrderedCollection new.
	resourceMutex := Semaphore forMutualExclusion.
	queueMutex := Semaphore forMutualExclusion.! !

Object subclass: #ALServer
	instanceVariableNames: 'state listener application connections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:13'!
initialize
	Smalltalk addToStartUpList: self after: AutoStart.
	Smalltalk addToShutDownList: self! !

!ALServer class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 17:55'!
on: anInteger application: anApplication
	^ self basicNew initializeWithPort: anInteger application: anApplication! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
shutDown: aBoolean
	self allInstancesDo: [:ea | ea shutDown]! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
startUp: aBoolean
	self allInstancesDo: [:ea | ea startUp]! !

!ALServer methodsFor: 'accessing' stamp: 'cwp 3/6/2012 16:01'!
application
	^ application! !

!ALServer methodsFor: 'private' stamp: 'cwp 10/19/2011 20:35'!
handleConnection: aSocket
	| connection |
	connection := ALConnection socket: aSocket application: application.
	connections add: connection.
	connection start! !

!ALServer methodsFor: 'initialize-release' stamp: 'cwp 10/19/2011 19:55'!
initializeWithPort: anInteger application: anApplication
	self initialize.
	listener := ALListener server: self port: anInteger.
	application := anApplication.
	connections := Set new.
	state := #stopped! !

!ALServer methodsFor: 'testing' stamp: 'cwp 4/13/2011 17:01'!
isRunning
	^ state == #running! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:16'!
printOn: aStream
	aStream nextPutAll: (self streamingPrintOn: String new writing)! !

!ALServer methodsFor: 'private' stamp: 'cwp 10/29/2012 00:21'!
purge
	
	(connections reject: [:ea | ea age < 60000 and: [ea isConnected]]) do:
		[:ea |
		ea stop.
		connections remove: ea].! !

!ALServer methodsFor: 'running' stamp: 'cwp 10/19/2011 01:02'!
shutDown
	state == #running
		ifTrue: [ 
			listener shutDown.
			state := #suspended ]! !

!ALServer methodsFor: 'running' stamp: 'cwp 10/19/2011 19:08'!
start
	self isRunning
		ifTrue: [ self stop ].
	state := #running.
	listener start! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:30'!
startUp
	state == #suspended ifTrue: [self start]! !

!ALServer methodsFor: 'running' stamp: 'cwp 10/27/2012 13:54'!
stop
	listener stop.
	connections do: [:ea | ea stop].
	state := #stopped! !

!ALServer methodsFor: 'printing' stamp: 'cwp 10/19/2011 00:44'!
streamingPrintOn: aWriteStream
	^ aWriteStream
		write: self class name;
		put: $[;
		print: listener port;
		put: $:;
		write: state;
		put: $];
		conclusion! !

Object variableSubclass: #ALStreamable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

ALStreamable variableSubclass: #ALPath
	instanceVariableNames: ''
	classVariableNames: 'Slash'
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/20/2011 22:57'!
/ aString
	^ self with: aString! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:16'!
initialize
	Slash := '/'! !

!ALPath class methodsFor: 'dispatch' stamp: 'cwp 10/25/2011 13:45'!
parseWith: aParser
	^ aParser parsePath! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:31'!
root
	^ self new: 0! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 3/11/2011 14:48'!
with: aString
	^ (self new: 1)
		at: 1 put: aString;
		yourself! !

!ALPath class methodsFor: 'as yet unclassified' stamp: 'cwp 8/16/2011 23:58'!
withAll: segments
	| path |
	path := self new: segments size.
	1 to: path size do: [:index | path at: index put: (segments at: index)].
	^ path! !

!ALPath methodsFor: 'building' stamp: 'cwp 9/13/2011 10:49'!
/ aString
	| path |
	(self isRoot and: [aString isEmpty]) ifTrue: [^ self].
	path := self class new: self size + 1.
	1 to: self size do: [:i | path at: i put: (self at: i)].
	path at: (path size) put: aString.
	^ path! !

!ALPath methodsFor: 'comparing' stamp: 'cwp 3/21/2011 22:20'!
= other
	self species = other species ifFalse: [^ false].
	self size = other size ifFalse: [^ false].
	1 to: self size do: [:i | (self at: i) = (other at: i) ifFalse: [^ false]].
	^ true! !

!ALPath methodsFor: 'converting' stamp: 'cwp 10/9/2012 00:18'!
asPath
	^ self! !

!ALPath methodsFor: 'converting' stamp: 'cwp 7/11/2012 22:57'!
asString
	^ self printString! !

!ALPath methodsFor: 'converting' stamp: 'cwp 10/9/2012 00:17'!
asUrl
	^ ALUrl withPath: self! !

!ALPath methodsFor: 'accessing' stamp: 'cwp 3/11/2011 15:17'!
first
	^ self at: 1! !

!ALPath methodsFor: 'comparing' stamp: 'cwp 3/21/2011 22:22'!
hash
	| hash |
	hash := self class hash.
	1 to: self size do: [:i | hash := hash bitXor: (self at: i) hash].
	^ hash! !

!ALPath methodsFor: 'testing' stamp: 'cwp 3/11/2011 14:54'!
isRoot
	^ self size = 0! !

!ALPath methodsFor: 'accessing' stamp: 'cwp 10/7/2012 23:31'!
path
	^ self! !

!ALPath methodsFor: 'accessing' stamp: 'cwp 3/11/2011 15:18'!
rest
	| rest |
	rest := self class new: self size - 1.
	1 to: rest size do: [:index | rest at: index put: (self at: index + 1)].
	^ rest! !

!ALPath methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:15'!
streamingPrintOn: aWriteStream
	self isRoot 
		ifTrue: [aWriteStream write: Slash]
		ifFalse:
			[1 to: self size do:
				[:index |
				aWriteStream write: Slash.
				aWriteStream write: (self at: index)]]! !

!ALStreamable class methodsFor: 'instance creation' stamp: 'cwp 10/4/2012 14:24'!
fromString: aString
	self subclassResponsibility! !

!ALStreamable class methodsFor: 'instance creation' stamp: 'cwp 10/4/2012 14:25'!
ofSize: anInteger
	^ self new: anInteger! !

!ALStreamable class methodsFor: 'streaming' stamp: 'cwp 10/4/2012 14:25'!
writing
	^ XTSequenceableCollectionWriteStream on: self new! !

!ALStreamable methodsFor: 'comparing' stamp: 'cwp 10/7/2012 23:49'!
= other
	self species = other species ifFalse: [^ false].
	self size = other size ifFalse: [^ false].
	1 to: self size do:
		[:index |
		(self at: index) = (other at: index) ifFalse: [^ false]].
	^ true! !

!ALStreamable methodsFor: 'streaming' stamp: 'cwp 10/4/2012 14:24'!
copyFrom: start to: stop 
	"This is copied from SequenceableCollection, so that instances can be created with a writeStream"

	| newSize |
	newSize := stop - start + 1.
	^(self species new: newSize)
		replaceFrom: 1
		to: newSize
		with: self
		startingAt: start! !

!ALStreamable methodsFor: 'streaming' stamp: 'cwp 10/4/2012 14:24'!
grownBy: length
	"This is copied from SequenceableCollection, so that instances can be created with a writeStream"

	| newCollection |
	newCollection := self species ofSize: self size + length.
	newCollection replaceFrom: 1 to: self size with: self startingAt: 1.
	^ newCollection! !

!ALStreamable methodsFor: 'comparing' stamp: 'cwp 10/7/2012 23:53'!
hash
	| hash |
	hash := self class identityHash.
	1 to: self size do:
		[:i | hash := ByteString stringHash: (self at: i) initialHash: hash].
	^ hash! !

!ALStreamable methodsFor: 'printing' stamp: 'cwp 10/4/2012 14:29'!
printOn: aStream
	| ws |
	ws := String new writing.
	self streamingPrintOn: ws.  
	aStream nextPutAll: ws conclusion.! !

!ALStreamable methodsFor: 'printing' stamp: 'cwp 10/4/2012 14:28'!
printString
	^ String new writing
		print: self;
		conclusion! !

!ALStreamable methodsFor: 'streaming' stamp: 'cwp 10/9/2012 00:20'!
reading
	^ XTSequenceableCollectionReadStream on: self! !

!ALStreamable methodsFor: 'streaming' stamp: 'cwp 10/4/2012 14:24'!
replaceFrom: start to: stop with: replacement startingAt: repStart 
	"Uses the Array primitive, so that instances can be created with a writeStream"

	<primitive: 105>
	! !

!ALStreamable methodsFor: 'streaming' stamp: 'cwp 10/7/2012 23:04'!
writing
	^ XTSequenceableCollectionWriteStream on: self! !

ALStreamable variableSubclass: #ALUrl
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALUrl class methodsFor: 'instance creation' stamp: 'cwp 10/7/2012 22:38'!
/ aString
	^ (self new: 2)
		at: 1 put: $/;
		at: 2 put: aString;
		yourself! !

!ALUrl class methodsFor: 'instance creation' stamp: 'cwp 10/4/2012 14:18'!
fromString: aString
	| ws |
	ws := self writing.
	aString reading percentDecoding streamingWriteInto: ws.
	^ ws conclusion! !

!ALUrl class methodsFor: 'dispatch' stamp: 'cwp 10/4/2012 14:36'!
parseWith: aParser
	^ aParser parseUrl! !

!ALUrl class methodsFor: 'instance creation' stamp: 'cwp 10/7/2012 23:30'!
root
	^ self with: $/! !

!ALUrl class methodsFor: 'instance creation' stamp: 'cwp 10/7/2012 23:30'!
with: anObject
	^ (self new: 1)
		at: 1 put: anObject;
		yourself! !

!ALUrl class methodsFor: 'instance creation' stamp: 'cwp 10/8/2012 00:13'!
withPath: aPath
	| out |
	aPath isRoot ifTrue: [^ self root].
	out := (self new: (aPath size * 2 - 1)) writing.
	1 to: aPath size do:
		[:index |
		out put: $/.
		out put: (aPath at: index)].
	^ out conclusion! !

!ALUrl methodsFor: 'arithmetic' stamp: 'cwp 10/7/2012 23:28'!
/ aString
	| out index token |
	out := (self class new: self size + 2) writing.
	index := 1.
	[(index > self size) or: ['?#' includes: (token := self at: index)]] 
		whileFalse: 
			[out put: token.
			index := index + 1].
	out put: $/.
	out put: aString.
	^ out conclusion! !

!ALUrl methodsFor: 'comparing' stamp: 'cwp 10/7/2012 23:58'!
hash
	"The handling of character tokens is taken from String class>>stringHash:initialHash"

	| hash |
	hash := self class identityHash.
	1 to: self size do: 
		[ :i || token low |
		token := self at: i.
		token isString
			ifTrue: [hash := ByteString stringHash: token initialHash: hash]
			ifFalse: 
				[hash := hash + token asciiValue.
				low := hash bitAnd: 16383.
				hash := (16r260D * low + ((16r260D * (hash bitShift: -14) 
						+ (16r0065 * low) bitAnd: 16383) * 16384)) bitAnd: 16r0FFFFFFF]].
	^ hash! !

!ALUrl methodsFor: 'accessing' stamp: 'cwp 10/7/2012 23:43'!
path
	| in out token seg |
	in := self reading.
	out := ALPath writing.
	seg := String new writing.
	[
		"Skip to the beginning of the path"
		[in get = $/] whileFalse.
		(token := in get) = $/ ifTrue: 
			[[in get = $/] whileFalse.
			token := in get]
			.			
		"Scan through the path"
		['?#' includes: token] whileFalse:
			[token = $/
				ifTrue: 
					[out put: seg conclusion.
					seg := String new writing]
				ifFalse:
					[seg write: token].
		token := in get].
	] on: Incomplete do: [].
	seg position = 0 ifFalse: [out put: seg conclusion].
	^ out conclusion! !

!ALUrl methodsFor: 'accessing' stamp: 'cwp 10/9/2012 00:34'!
query
	| in out |
	query := Array new.
	in := self reading ending: $#.
	[ [ in get = $? ] whileFalse ]
		on: Incomplete
		do: [ ^ query ].
	out := query writing.
	^ (ALFormDataParser in: in out: out) parse! !

!ALUrl methodsFor: 'printing' stamp: 'cwp 10/8/2012 14:19'!
streamingPrintOn: anAsciiWriteStream
	| enc |
	enc := anAsciiWriteStream percentEncoding.
	1 to: self size do: 
		[:i || token |
		token := self at: i.
		token ifNotNil:
			[token isString
				ifTrue: [enc write: token]
				ifFalse: [anAsciiWriteStream put: token]]]! !

Object subclass: #ALTerminalRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Relays'!

!ALTerminalRelay methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:08'!
destination: anObject
	self error: 'Terminal relays have no destination'! !

!ALTerminalRelay methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:25'!
handleRequest: aRequest
	^ ALResponse notFound: aRequest url! !

!ALTerminalRelay methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:19'!
write: aMessage into: aStream
	aMessage writeBodyInto: aStream! !

Object subclass: #ALWorker
	instanceVariableNames: 'process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

ALWorker subclass: #ALConnection
	instanceVariableNames: 'application in queue timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALConnection class methodsFor: 'as yet unclassified' stamp: 'cwp 4/24/2011 23:17'!
in: aReadStream out: aWriteStream application: anApplication
	^ self basicNew 
		initializeWithIn: aReadStream 
		out: aWriteStream 
		application: anApplication ! !

!ALConnection class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 23:17'!
socket: aSocket application: anApplication
	^ self in: aSocket reading out: aSocket writing application: anApplication! !

!ALConnection methodsFor: 'accessing' stamp: 'cwp 11/1/2012 22:45'!
age
	^ Time millisecondClockValue - timestamp! !

!ALConnection methodsFor: 'accessing' stamp: 'cwp 11/1/2012 23:05'!
close
	self socket closeAndDestroy! !

!ALConnection methodsFor: 'private' stamp: 'cwp 11/1/2012 23:04'!
handleRequest
	| request |
	request := ALRequest readFrom: in.
	[ self respondTo: request ] 
		forkAt: Processor highIOPriority
		named: 'dispatch:' , self identityHash printString! !

!ALConnection methodsFor: 'initialize-release' stamp: 'cwp 11/1/2012 22:53'!
initializeWithIn: aReadStream out: aWriteStream application: anApplication
	
	self initialize.
	application := anApplication.
	in := aReadStream.
	queue := ALResourceQueue for: aWriteStream.
	self touch
! !

!ALConnection methodsFor: 'testing' stamp: 'cwp 11/1/2012 22:51'!
isConnected
	^ self socket isConnected! !

!ALConnection methodsFor: 'running' stamp: 'cwp 11/1/2012 22:49'!
loop
	[self wait.
	 self handleRequest ] 
		repeat! !

!ALConnection methodsFor: 'accessing' stamp: 'cwp 11/1/2012 23:07'!
name
	| addr socket |
	socket := self socket.
	addr := socket isConnected
		ifTrue: [socket remoteAddress printString]
		ifFalse: ['<closed>'].
	
	^ 'connection: ' , addr! !

!ALConnection methodsFor: 'accessing' stamp: 'cwp 11/1/2012 22:48'!
priority
	^ Processor highIOPriority! !

!ALConnection methodsFor: 'private' stamp: 'cwp 11/1/2012 22:57'!
respondTo: request
	| response |
	queue add.
	response := application handleRequest: request.
	queue do: [ :out | response writeInto: out ].
	self touch! !

!ALConnection methodsFor: 'accessing' stamp: 'cwp 11/1/2012 22:48'!
socket
	^ in terminal! !

!ALConnection methodsFor: 'running' stamp: 'cwp 11/1/2012 22:49'!
tearDown
	self close! !

!ALConnection methodsFor: 'accessing' stamp: 'cwp 11/1/2012 22:45'!
touch
	timestamp := Time millisecondClockValue! !

!ALConnection methodsFor: 'private' stamp: 'cwp 11/1/2012 22:50'!
wait
	
	[self socket waitForDataIfClosed: 
		[ConnectionClosed signal: 'Connection closed.']]
		on: SocketPrimitiveFailed
		do: [:e | .ConnectionClosed signal: 'Connection closed.'].
	self touch! !

ALWorker subclass: #ALListener
	instanceVariableNames: 'server port socket'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALListener class methodsFor: 'as yet unclassified' stamp: 'cwp 10/19/2011 19:05'!
server: aServer port: anInteger
	^ self basicNew initializeWithServer: aServer port: anInteger! !

!ALListener methodsFor: 'private' stamp: 'cwp 10/19/2011 19:07'!
acceptConnection
	| sock |
	sock := socket waitForAcceptFor: self timeout.
	sock ifNotNil: [ server handleConnection: sock ]! !

!ALListener methodsFor: 'initialize-release' stamp: 'cwp 10/19/2011 19:01'!
initializeWithServer: aServer port: anInteger
	self initialize.
	server := aServer.
	port := anInteger! !

!ALListener methodsFor: 'running' stamp: 'cwp 10/19/2011 19:55'!
loop
	[self acceptConnection.
	server purge] 
		repeat! !

!ALListener methodsFor: 'accessing' stamp: 'cwp 11/1/2012 23:08'!
name
	^ 'listener: ', port printString! !

!ALListener methodsFor: 'accessing' stamp: 'cwp 10/19/2011 00:15'!
port
	^ port! !

!ALListener methodsFor: 'accessing' stamp: 'cwp 10/26/2012 23:50'!
priority
	^ Processor lowIOPriority! !

!ALListener methodsFor: 'running' stamp: 'cwp 10/26/2012 23:42'!
setUp
	socket := Socket tcpCreateIfFail: [ self error: 'Unable to create a socket' ].
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: 1024;
		setOption: 'SO_RCVBUF' value: 1024;
		listenOn: self port backlogSize: 10.
	socket isValid
		ifFalse: [ self error: 'Unable to listen on port ' , self port printString ]! !

!ALListener methodsFor: 'running' stamp: 'cwp 10/26/2012 23:58'!
tearDown
	socket ifNotNil: 
		[socket closeAndDestroy.
		socket := nil]! !

!ALListener methodsFor: 'accessing' stamp: 'cwp 10/19/2011 00:33'!
timeout
	^ 5! !

!ALWorker methodsFor: 'accessing' stamp: 'cwp 10/27/2012 00:04'!
exceptions
	^ ConnectionClosed! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/26/2012 23:53'!
loop
	self subclassResponsibility! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/30/2012 17:44'!
run
	self setUp.
	[[self loop] 
		on: self exceptions
		do: [:e]]
		ensure: 
			[self tearDown.
			process := nil]! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/26/2012 23:54'!
setUp
! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/26/2012 23:51'!
shutDown
	process ifNotNil: [ process terminate ]! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/26/2012 23:52'!
start
	process := [self run] 
		forkAt: self priority 
		named: self name! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/26/2012 23:52'!
stop
	process ifNotNil: [ process terminate ]! !

!ALWorker methodsFor: 'running' stamp: 'cwp 10/26/2012 23:54'!
tearDown! !

Array variableSubclass: #ALCookieSpecs
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Values'!

!ALCookieSpecs class methodsFor: 'dispatch' stamp: 'cwp 7/12/2012 16:02'!
parseWith: aParser
	^ aParser parseCookieSpecs! !

!ALCookieSpecs methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 13:09'!
asCookieSpecs
	^ self! !

!ALCookieSpecs methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 13:10'!
at: anInteger put: aCookie
	(aCookie isKindOf: ALCookie) ifFalse: [self error: 'Not a cookie'].
	super at: anInteger put: aCookie! !

!ALCookieSpecs methodsFor: 'printing' stamp: 'cwp 7/12/2012 15:42'!
printWith: aPrinter
	aPrinter printCommaSequence: self! !

Array variableSubclass: #ALCookieValues
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Values'!

!ALCookieValues class methodsFor: 'dispatch' stamp: 'cwp 7/12/2012 15:35'!
parseWith: aParser
	^ aParser parseCookieValues! !

!ALCookieValues methodsFor: 'printing' stamp: 'cwp 7/12/2012 15:50'!
printWith: aPrinter
	aPrinter printSemicolonSequence: self! !

Error subclass: #ALHttpError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

ALHttpError subclass: #ALMalformedRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

!ALMalformedRequest methodsFor: 'as yet unclassified' stamp: 'cwp 3/14/2012 13:26'!
handleWith: anObject
	^ anObject handleMalformedRequest: self! !

ALHttpError subclass: #ALMethodNotAllowed
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

!ALMethodNotAllowed methodsFor: 'as yet unclassified' stamp: 'cwp 3/31/2012 20:17'!
handleWith: anObject
	^ anObject handleMethodNotAllowed: self! !

ALHttpError subclass: #ALNotAcceptable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

!ALNotAcceptable methodsFor: 'as yet unclassified' stamp: 'cwp 3/14/2012 13:26'!
handleWith: anObject
	^ anObject handleNotAcceptable: self.! !

ALHttpError subclass: #ALNotFound
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

!ALNotFound methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2008 23:01'!
handleWith: anObject
	^ anObject handleNotFound: self! !

ALHttpError subclass: #ALUnsupportedContentType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

!ALUnsupportedContentType methodsFor: 'as yet unclassified' stamp: 'cwp 10/14/2011 23:50'!
handleWith: anObject
	^ anObject handleUnsupportedContentType: self! !

Error subclass: #ALParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Exceptions'!

!Number methodsFor: '*al-http-printing' stamp: 'cwp 10/25/2011 14:59'!
printWith: aPrinter
	aPrinter printNumber: self! !

!ByteArray methodsFor: '*al-http-printing' stamp: 'cwp 10/25/2011 14:22'!
printWith: aPrinter
	aPrinter printBytes: self! !
ALHttpInterpreter initialize!
ALServer initialize!
ALPath initialize!
